# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Reminders app autopilot tests."""

import os
import os.path
import logging
import sqlite3
import shutil

from autopilot.input import Mouse, Touch, Pointer
from autopilot.platform import model
from autopilot.testcase import AutopilotTestCase
from testtools.matchers import Equals
from autopilot.matchers import Eventually

from ubuntuuitoolkit import (
    base,
    emulators as toolkit_emulators
)

from reminders_app import emulators

logger = logging.getLogger(__name__)


def get_module_include_path():
    return os.path.abspath(
        os.path.join(
            os.path.dirname(__file__),
            '..',
            '..',
            '..',
            '..',
            'builddir/src/plugin/')
        )


class RemindersAppTestCase(AutopilotTestCase):
    """A common test case class that provides several useful methods for
       reminders-app tests."""

    if model() == 'Desktop':
        scenarios = [('with mouse', dict(input_device_class=Mouse))]
    else:
        scenarios = [('with touch', dict(input_device_class=Touch))]

    local_location = "../../src/app/qml/reminders-app.qml"
    installed_location = "/usr/share/reminders-app/qml/reminders-app.qml"

    def setUp(self):
        self.pointing_device = Pointer(self.input_device_class.create())
        super(RemindersAppTestCase, self).setUp()

        if os.path.exists(self.local_location):
            self.launch_test_local()
        elif os.path.exists(self.installed_location):
            self.launch_test_installed()
        else:
            self.launch_test_click()

    def launch_test_local(self):
        self.app = self.launch_test_application(
            base.get_qmlscene_launch_command(),
            "-I", get_module_include_path(),
            self.local_location,
            "--desktop_file_hint=/home/phablet/reminders-app/"
            "reminders-app.desktop",
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_installed(self):
        self.app = self.launch_test_application(
            base.get_qmlscene_launch_command(),
            self.installed_location,
            "--desktop_file_hint=/usr/share/applications/"
            "reminders-app.desktop",
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_click(self):
        self.app = self.launch_click_package(
            "com.ubuntu.reminders-app",
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    @property
    def main_view(self):
        return self.app.select_single(emulators.MainView)

class DatabaseMixin(object):

    """
    Helper functions for dealing with sqlite databases
    """

    app_dir = os.path.expanduser(
            "~/.local/share/com.ubuntu.reminders-app")
    db_dir = os.path.join(app_dir, 'Databases')
    db_file = "34e1e542f2f083ff18f537b07a380071.sqlite"
    db_path = os.path.join(db_dir, db_file)

    online_acct_dir = os.path.expanduser(
                    "~/.config/libaccounts-glib")
    online_acct_db_file = "accounts.db"
    online_acct_db_path = os.path.join(online_acct_dir, online_acct_db_file)

    def _execute_sql(self, statement, db=""):
        if not db:
            db = self.online_acct_db_path
        conn = sqlite3.connect(db)
        cursor = conn.cursor()
        logger.debug("Executing sql")
        logger.debug(statement)
        cursor.execute(statement)
        conn.commit()
        conn.close()

    def backup_database(self, db=""):
        if not db:
            db = self.online_acct_db_path
        db_backup = db + ".bck"
        logger.debug("Backing up existing database at %s to %s" % (db, db_backup))

        #remove backup db
        self.remove_db(db_backup)

        try:
            shutil.move(db, db_backup)
        except OSError:
            logger.debug("Could not backup existing database")
            pass

    def restore_database(self, db=""):
        if not db:
            db = self.online_acct_db_path
        db_backup = db + ".bck"
        logger.debug("Restoring pre-existing database from %s to %s" % (db_backup, db))

        #remove test db
        self.remove_db()

        try:
            shutil.move(db_backup, db)
        except OSError:
            logger.debug("Could not restore pre-existing database")
            pass

    def remove_db(self, db=""):
        if not db:
            db = self.online_acct_db_path
        logger.debug("Removing existing database at %s" % db)
        try:
            os.remove(db)
        except OSError:
            logger.debug("Could not remove existing database")
            pass

    def create_blank_db(self):
        self.remove_db()

        #create blank db
        logger.debug("Creating blank db on filesystem %s" % self.online_acct_db_path)
        if not os.path.exists(self.online_acct_dir):
            os.makedirs(self.online_acct_dir)
        open(self.online_acct_db_path, 'a').close()
        self.assertThat(
            lambda: os.path.exists(self.online_acct_db_path),
            Eventually(Equals(True)))

        #this needs to stay in sync with online accounts
        logger.debug("Creating accounts and settings tables")
        self._execute_sql("CREATE TABLE IF NOT EXISTS Accounts (id INTEGER PRIMARY KEY AUTOINCREMENT,name TEXT,provider TEXT,enabled INTEGER)")
        self._execute_sql("CREATE TABLE IF NOT EXISTS Settings (account INTEGER NOT NULL,service INTEGER,key TEXT NOT NULL,type TEXT NOT NULL,value BLOB)")
        self._execute_sql("CREATE TABLE Services (id INTEGER PRIMARY KEY AUTOINCREMENT,name TEXT NOT NULL UNIQUE,display TEXT NOT NULL,provider TEXT,type TEXT)")
        self._execute_sql("INSERT INTO Services VALUES(36,'evernote','Evernote','evernote','sharing')")

        #not needed
        #self._execute_sql("CREATE TABLE Signatures (account INTEGER NOT NULL,service INTEGER,key TEXT NOT NULL,signature TEXT NOT NULL,token TEXT NOT NULL)")

    def add_account_to_database(self):
        logger.debug("Adding mockaccount to database")
        self._execute_sql("INSERT INTO Accounts VALUES(13,'mockaccount','evernote',1)")
        self._execute_sql("INSERT INTO Settings VALUES(13,0,'CredentialsId','u',24)")
        self._execute_sql("INSERT INTO Settings VALUES(13,0,'name','s','''mockaccount''')")
        self._execute_sql("INSERT INTO Settings VALUES(13,36,'enabled','b','true')")
        self._execute_sql("INSERT INTO Settings VALUES(13,0,'enabled','b','true')")




